#==============================================================================
#
#  $Id: main.py,v 1.9 2000/06/16 00:03:32 mike Exp $
#
"""
   This module will ultimately be just a place where all of the stuff that is 
   needed from everywhere to be loaded into.  For now, it houses some of
   that stuff.
"""
#
#  Copyright (C) 1999 Michael A. Muller
#
#  Permission is granted to use, modify and redistribute this code,
#  providing that the following conditions are met:
#
#  1) This copyright/licensing notice must remain intact.
#  2) If the code is modified and redistributed, the modifications must 
#  include documentation indicating that the code has been modified.
#  3) The author(s) of this code must be indemnified and held harmless
#  against any damage resulting from the use of this code.
#
#  This code comes with ABSOLUTELY NO WARRANTEE, not even the implied 
#  warrantee of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#
#  $Log: main.py,v $
#  Revision 1.9  2000/06/16 00:03:32  mike
#  Added the "entry menu" widget, used it for folder selection.
#
#  Revision 1.8  2000/05/11 01:01:33  mike
#  Menus now call command functions _after_ repainting the owner.
#
#  Revision 1.7  2000/04/30 21:07:22  mike
#  Added initial address book window.
#
#  Revision 1.6  2000/04/27 23:18:12  mike
#  Took out a lot of code that now lives elsewhere.
#
#  Revision 1.5  2000/04/17 12:34:32  mike
#  Load behavior if no custom.
#
#  Revision 1.4  2000/04/14 01:21:13  mike
#  Added the post office global and changed menu to allow values as well as
#  commands
#
#  Revision 1.3  2000/04/12 00:43:54  mike
#  Added hidden entryfields and initial support for customization.
#
#  Revision 1.2  2000/04/02 22:38:33  mike
#  sendMessage() now really works.
#
#  Revision 1.1.1.1  2000/04/02 19:09:03  mike
#  Spug Mailer and libraries.
#
#
#==============================================================================

# get the user's spugmailrc file
import os
home = os.environ['HOME']
rcfile = os.path.join(home, '.spugmailrc')
custom = None
if os.path.exists(rcfile):
   glbls = {}
   execfile(rcfile, glbls)
   if glbls.has_key('custom'):
      custom = glbls['custom']

# if something went wrong, create an instance of Behavior
if not custom:
   from behavior import Behavior
   custom = Behavior()

# create the "post office"
import mailbx
postOffice = mailbx.PostOffice(custom.getFolderPath())

# load the address book
import addrbk
addressBook = addrbk.AddressBook(custom.getAddressBook())

# finally, import the main window module
from mainwin import mainWin

#------------------------------------------------------------------------------
# GUI Toys
#------------------------------------------------------------------------------

import string
from spug.curses.KeyEvent import KeyEvent
from spug.curses.ModalWindow import ModalWindow
from spug.curses.Listbox import Listbox
from spug.curses.Control import Control

class _MenuItem:
   
   def __init__(self, item, action):
      self.item = item
      self.action = action

class _MenuList(Listbox):
   
   def _itemAsString(self, item):
      return item.item
   
class _MenuBase(ModalWindow):
   """
      Menu base class: used by Menu and EntryMenu.
   """

   def paint(self):
      self.box()
      self.refresh()
      self.list.paint()
   
   def __init__(self, items, x, y, width, height, listOffset):

      # holds the value selected (if the menu does not process commands)
      self.value = None

      # variable to compute the maximum width of any display string in the menu
      maxWidth = 0
      
      # compute the maximum width of the menu
      for elem in items:      
      
         # check to see if we need a wider menu         
         lenDispStr = len(elem.item)
         if lenDispStr > maxWidth:
            maxWidth = lenDispStr
      
      # check to see if the menu is greater than the parent width, shrink it
      # if it is
      if maxWidth + 2 > width:
         maxWidth = width - 2
      
      # check so see if the menu height is greater than the parent height,
      # shrink if necessary.
      if len(items) + 2 > height:
         myHeight = height
      else:
         myHeight = len(items) + 2 + listOffset
      
      # finally, create the ModalWindow
      ModalWindow.__init__(self, x, y, maxWidth + 2, myHeight)

      # create the list control
      self.list = _MenuList(self, 1, 1 + listOffset, self._width - 2, 
                            self._height - (2 + listOffset),
                            items
                            )
      
      self.paint()

   def processEvent(self, evt):
   
      # first delegate to ModalWindow so the list can have a shot at it
      if ModalWindow.processEvent(self, evt):
         return 1
      
      if isinstance(evt, KeyEvent):
         if evt.code == 10:
            action = self.list.selection().action
            self.value = action
            self.terminate()
            return 1
            
      return 0

class Menu(_MenuBase):
   
   def __init__(self, items, x, y, width, height):
   
      # a dictionary mapping keycodes to functions to call
      self.funcs = {}
   
      # list items
      listItems = []

      # build all of the info for the menu items
      for code, dispStr, func in items:
         
         # add to the key map
         self.funcs[code] = func
         
         # add to the menu list
         listItems.append(_MenuItem(dispStr, func))

      _MenuBase.__init__(self, listItems, x, y, width, height, 0)

   def processEvent(self, evt):

      # let the base class try to deal with it first
      if _MenuBase.processEvent(self, evt):
         return 1
   
      if isinstance(evt, KeyEvent):
         if self.funcs.has_key(evt.code):
            action = self.funcs[evt.code]
            self.value = action
            self.terminate()
         else:
            self.beep()
            self.terminate()
         return 1
      else:
         return 0

class _MiniEntryfield(Control):
   
   def __init__(self, parent, x, y, width, height, list):
      Control.__init__(self, parent, x, y, width, height)
      
      self.__text = ''
      self.__list = list
   
   def processEvent(self, evt):
      if isinstance(evt, KeyEvent):
         asChr = evt.code < 256 and chr(evt.code) or '.'
         if asChr in string.letters:
            self.__findItem(self.__text + asChr)
            return 1
         elif evt.code == self.KEY_BACKSPACE:
            self.__findItem(self.__text[:-1])
            return 1
         else:
            return 0
      else:
         return 0

   def __findItem(self, text):
      items = self.__list.getItems()
      text = string.lower(text)
      for item in items:
         if string.lower(item.item)[:len(text)] == text:
            self.__list.setSelection(item)
            self.__list.paint()
            self.__text = text
            self.paint()
            self.refresh()
            return 1
      else:
         self.beep()
         self.refresh()
         return 0            

   def paint(self):
      self.writeAt(0, 0, self.__text + (self._width - len(self.__text)) * ' ')
      

class EntryMenu(_MenuBase):
   """
      This class is really more of a drop-down box with name completion.
      It displays a menu with a blank line at the top.  Alphabetic
      keys are used to move the `menu/list` selection bar to the first
      selection prefaced by the substring.
   """
   
   def __init__(self, items, x, y, width, height):
      """
         parms:
            items::
               [list<tuple<string, any>>] list of tuples: first item is the
               display string, second is the value.  As in @Menu, if the
               item is callable, it will be executed upon selection.
            x::
               [int] x pos.
            y::
               [int] y pos.
            width::
               [int] parent window width.
            height::
               [int] parent window height.
      """
      
      # first, sort the item list, ignore case
      items.sort(lambda a, b: cmp(string.lower(a[0]), string.lower(b[0])))
      
      # create the set of list items
      listItems = map(lambda x: _MenuItem(x[0], x[1]), items)
      
      # initialize the base class, allow one row at the top
      _MenuBase.__init__(self, listItems, x, y, width, height, 1)
      
      # add the tiny-entry widget
      self.__miniEF = _MiniEntryfield(self, 1, 1, self._width - 2,
                                      self._height - 2,
                                      self.list
                                      )
      self.addChild(self.__miniEF)
      
      self.setFocus(self.list)

   def processEvent(self, evt):
      """
         Overrides the processEvent method.
      """
      
      if isinstance(evt, KeyEvent):
         
         # check to see if this is one of the events that must be processed
         # by the mini entryfield
         if evt.code < 256 and chr(evt.code) in string.letters or \
            evt.code == self.KEY_BACKSPACE:
            return self.__miniEF.processEvent(evt)
         
         # otherwise send it to the modal window
         if _MenuBase.processEvent(self, evt):
            return 1
         else:
            self.beep()
            self.refresh()
            self.terminate()
            return 0
         
      else:
         return 0

def _anyMenu(items, parent, menuClass):
   w, h = parent.getSize()
   x, y = parent.getPos()
   menu = menuClass(items, x, y, w, h)
   menu.show(parent)
   val = menu.value
   if callable(val):
      val()
   else:
      return val
      
def menu(items, parent):
   """
      If the selected menu item's value is a callable object, it will be 
      executed instead of returned.
      parms:
         items::
            [list<tuple<int, string, any>>] A list of tuples indicating
            the key code, display string, and value of each item
            in the menu.  If the value is a callable object, it will be
            executed when the menu item is selected.  Any other kind of
            object will be returned.
         parent::
            [@curses.Toplevel.Toplevel] Toplevel window to repaint when
            done processing
   """
   return _anyMenu(items, parent, Menu)
   
def entryMenu(items, parent):
   """
      parms:
         items::
            [list<tuple<string, any>>] list of tuples: first item is the
            display string, second is the value.  As in @Menu, if the
            item is callable, it will be executed upon selection.
         parent::
            [@curses.Toplevel.Toplevel] Toplevel window to repaint when
            done processing
   """
   return _anyMenu(items, parent, EntryMenu)
